VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "SyncLogic"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Ch6bLogic.cls
' Copyright 2001 Matthew J. Holmes
'
' This module provides the synchronization logic for the conduit. This is a stripped
' down conduit that does nothing but pull raw records from the Palm and convert them
' into Windows files.
'
' The records are fielded, with data in the following format:
'
'     Variable: Date  Boolean Long  String
'     Offsets:  |0..3|4       |5..8|9..28|
'

Option Explicit
   
' Synchonize() - dispatch supported hot-sync requests
'
Public Sub Synchronize()

    ' Get the HotSync information object for this conduit
    Dim pdSys As New PDSystemAdapter
    Dim pdHSInfo As PDHotsyncInfo
    Set pdHSInfo = pdSys.PDHotsyncInfo
    
    ' Route the requested synchronization to the local handler
    Select Case pdHSInfo.SyncType
        Case eHHtoPC
            HHtoPC pdSys, pdHSInfo
        Case Else
            LogSync pdSys, pdHSInfo.SyncType
    End Select

End Sub

' Log the sync based on the type
Private Sub LogSync(ByRef pdSys As PDSystemAdapter, _
                    ByVal SyncType As ESyncTypes)
                       
    Dim strType As String
    
    Select Case SyncType
    Case eFast
        strType = "Fast"
    Case eSlow
        strType = "Slow"
    Case ePCtoHH
        strType = "PC-to-HH"
    Case eHHtoPC
        strType = "HH-to-PC"
    Case eInstall
        strType = "Install"
    Case eBackup
        strType = "Backup"
    Case eDoNothing
        strType = "Nothing"
    Case eProfileInstall
        strType = "Profile Install"
    Case Else
        strType = "Unknown"
    End Select
    
    pdSys.AddLogEntry "Ch6b - " + strType, eText, False, False
    
End Sub

' HHtoPC() -  Blow away all files in the configured directory and
' refresh all records from the Palm.
Private Sub HHtoPC(ByRef pdSys As PDSystemAdapter, _
                   ByRef pdHSInfo As PDHotsyncInfo)

    ' Data is under user's directory in Hot-Sync area.
    Dim DBPath As String
    DBPath = pdHSInfo.PathName + "Ch4b"
    
    ' Purge the PC data - force a new directory if necessary.
    Dim FSO As New FileSystemObject
    On Error Resume Next
    FSO.DeleteFile DBPath + "\*.*", True
    FSO.CreateFolder DBPath
    On Error GoTo 0
    
    ' Get the Palm database from the Hot-Sync manager
    Dim DBName As String
    DBName = pdHSInfo.NameList(0)
    
    Dim pdQuery As New PDDatabaseQuery
    Dim pdRecords As PDRecordAdapter
    Set pdRecords = pdQuery.OpenRecordDatabase(DBName, "PDDirect.PDRecordAdapter")
    
    ' Open the handheld database, and iterate over the records
    Dim Index As Long
    Dim RecordId As Variant
    Dim Category As Long
    Dim Attributes As ERecordAttributes
    Dim Data As Variant

    pdRecords.IterationIndex = 0
    Data = pdRecords.ReadNext(Index, RecordId, Category, Attributes)
    Do While Not pdRecords.EOF
    
        ' In a HHtoPC sync, process all but deleted records
        If Not CBool(CByte(Attributes) And CByte(eDelete)) Then
            WriteRecContents DBPath, RecordId, Data
        End If
 
        ' Read the next record and skip to the top of the loop
        Data = pdRecords.ReadNext(Index, RecordId, Category, Attributes)
    Loop
    
    '  Remove any deleted records, clear  flags in Palm database
    pdRecords.RemoveSet eRemoveAllDeletedRecords
    pdRecords.ResetAllModifiedFlags
    
    LogSync pdSys, pdHSInfo.SyncType

End Sub

Private Sub WriteRecContents(ByVal DBPath As String, _
                             ByRef RecordId As Variant, _
                             ByRef Data As Variant)
                             
    Dim Filenum As Integer
    Dim Filename As String
    Dim pdUtil As New PDUtility

    ' Convert the unique record identifier and use as filename
    Filename = DBPath + "\" + pdUtil.RecordIdToString(RecordId) + ".REC"
    Filenum = FreeFile
    
    ' Write the data, which we first convert to Unicode
    Open Filename For Output As #Filenum

    ' Get the date
    Dim d As Long
    pdUtil.ByteArrayToDWORD Data, 0, True, d
    Write #Filenum, CStr(PalmLongToDate(d))
    
    ' Get the time
    Dim T As Long
    pdUtil.ByteArrayToDWORD Data, 4, True, T
    Write #Filenum, CStr(PalmLongToTime(T))
    
    ' Get the boolean
    Dim b As Boolean
    b = Data(8)
    Write #Filenum, b
    
    ' Get the integer
    Dim i As Integer
    pdUtil.ByteArrayToWORD Data, 9, True, i
    Write #Filenum, i
    
    ' Get the long
    Dim l As Long
    pdUtil.ByteArrayToDWORD Data, 11, True, l
    Write #Filenum, l

    ' Get the string
    Dim s As String
    pdUtil.ByteArrayToBSTR Data, 15, UBound(Data) - LBound(Data) + 1 - 15, s
    Write #Filenum, s
    
    ' All done
    Close #Filenum
    
End Sub

Private Function PalmLongToDate(ByVal d As Long) As Date

    Dim SecsSince1904 As Double
    Dim DaysSince1904 As Double

    Const SecsPerDay As Long = 86400
    Const UnsignedLngMax As Double = 4294967296#
    
    ' Handle signed/unsigned issues and use a double to prevent overflow.
    If d < 0 Then
        SecsSince1904 = UnsignedLngMax + d
    Else
        SecsSince1904 = d
    End If
    
    ' Figure out how many days have passed since 1904. Then add to
    ' earliest possible date. Let VB adjust for leap year, etc!
    DaysSince1904 = SecsSince1904 / SecsPerDay
    PalmLongToDate = DateSerial(1904, 1, 1) + CLng(DaysSince1904)
    
End Function

Private Function PalmLongToTime(ByVal T As Long) As Date

    Dim Hours As Integer
    Dim Minutes As Integer
    Dim Seconds As Integer
    
    ' Strip off any vestigal seconds, handle signed/unsigned issues.
    If T < 0 Then T = T + &H10000
    T = T And &H1FFFF
    
    ' Calculate hours, minutes and seconds based
    Hours = T \ 3600
    Minutes = (T \ 60) Mod 60
    Seconds = T Mod 60

    PalmLongToTime = TimeSerial(Hours, Minutes, Seconds)
    
End Function

